import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';


export default class LineChartComponent extends Component {
  @tracked lineElement;

  get isLoaded() {
    return (this.args.dataset || this.args.dataset === []) && !!this.args.dimensions;
  }

  get fill() {
    return this.args.fill ? this.args.fill : 'none'
  }

  get fillOpacity() {
    return this.args.fillOpacity || '0.5'
  }

  get color() {
    return this.args.color ? this.args.color : 'black'
  }

  get strokeWidth() {
    return this.args.strokeWidth ? this.args.strokeWidth : '2'
  }

  get xKey() {
    return this.args.xKey ? this.args.xKey : 'at';
  }

  get yKey() {
    return this.args.yKey ? this.args.yKey : 'value';
  }

  get fillIn() {
    return this.fill !== 'none'
  }

  // If a filled in line graph (aka svg polygon) where do we draw the floor?
  get graphBottom() {
    return this.args.graphBottom || -10;
  }

  get currentLineInfo() {
    if(!this.isLoaded) return {}
    return this.fillIn
      ? this.generateSVGPolygon(this.args.xScale, this.args.yScale, this.args.dataset)
      : this.generateSVGLine(this.args.xScale, this.args.yScale, this.args.dataset)
  }

  generateSVGLine(xScale, yScale, dataset) {
    const xKey = this.xKey;
    const yKey = this.yKey;
    const firstX = dataset[0] ? dataset[0][xKey] : 0;
    const firstY = dataset[0] ? dataset[0][yKey] : 0;
    return `M${xScale(new Date(firstX))} ${yScale(firstY)} 
            ${dataset.slice(1).map(d => {
              const x = (typeof d[xKey] === "object") ? d[xKey] : new Date(d[xKey])
              return `L${xScale(x)} ${yScale(d[yKey])}`;
            }).join(' ')}`
  };

  generateSVGPolygon(xScale, yScale, dataset) {
    const {xKey, yKey} = this;
    const points = this.addPolygonDatapoints(dataset)
    return points.map((point) => {
      if(point[xKey] == null) return ""
      const foramttedDate = typeof point[xKey] === 'string' ? new Date(point[xKey]) : point[xKey]
      return point[yKey] || point[yKey] === 0 ? `${xScale(foramttedDate)},${yScale(point[yKey])}` : "";
    }).join(' ')
  }

  addPolygonDatapoints(dataset) {
    if(!dataset || dataset.length === 0) return [];
    const bottomLeftCorner = {};
    const bottomRightCorner = {};
    bottomLeftCorner[this.xKey] = dataset[0][this.xKey];
    bottomLeftCorner[this.yKey] = this.graphBottom;

    bottomRightCorner[this.xKey] = dataset[dataset.length - 1][this.xKey]
    bottomRightCorner[this.yKey] = this.graphBottom;

    return [
      bottomLeftCorner,
      ...dataset,
      bottomRightCorner
    ]
  }
}